# python 3 headers, required if submitting to Ansible
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = """
        lookup: nextcloud_plugins
        author: Juergen Haas <juergen.haas@lakedrops.com>
        version_added: "0.9"
        short_description: get a list of available nextcloud plugins
        description:
            - This lookup returns a list of relevant plugins and indicates which of them need to be downloaded first.
        options:
          _nc_versions:
            description: Nextcloud version(s)
            required: True
"""
from ansible.errors import AnsibleError, AnsibleParserError
from ansible.plugins.lookup import LookupBase
from pkg_resources import parse_version
import requests

try:
    from __main__ import display
except ImportError:
    from ansible.utils.display import Display
    display = Display()


class LookupModule(LookupBase):

    def run(self, nc_versions, variables=None, **kwargs):
        files = variables['download_list']['stdout_lines']
        install = variables['nextcloud_plugins']
        pinned = variables['nextcloud_pinned_plugin_versions']
        plugins = []
        for version in nc_versions:
            display.debug("Nextcloud version %s" % version)
            display.vvvv(u"Nextcloud version %s" % version)
            try:
                r = requests.get('https://apps.nextcloud.com/api/v1/platform/%s/apps.json' % version)
                for plugin in r.json():
                    if plugin['id'] in install:
                        install.remove(plugin['id'])
                        pinned_version = False
                        if plugin['id'] in pinned:
                            pinned_version = pinned[plugin['id']]
                        latest_version = '0.0.0'
                        latest_release = False
                        for release in plugin['releases']:
                            if pinned_version:
                                if pinned_version == release['version']:
                                    latest_release = release
                            else:
                                if parse_version(release['version']) > parse_version(latest_version):
                                    latest_version = release['version']
                                    latest_release = release
                        if latest_release:
                            filename = '%s-%s.tar.gz' % (plugin['id'], latest_release['version'])
                            plugin['needs_download'] = (not filename in files)
                            plugin['latest_release'] = latest_release
                            plugins.append(plugin)

            except AnsibleParserError:
                raise AnsibleError("could not download plugins for version %s" % version)

        for missing in install:
            plugins.append({
                "id": missing,
                "needs_download": 0,
                "missing": "yes"
            })

        if len(plugins) > 1:
            return plugins
        return [plugins]
